@@ -181,6 +181,9 @@ typedef struct
 	// True if it is a SAM slot
 	int isSamSlot;
 
+	/* TRUE if SAM notification is enabled, otherwise FALSE. */
+	int isSamNotified;
+
 	/* Write delay in milliseconds */
 	int writeDelay;
 } _ccid_descriptor;
@@ -282,6 +285,7 @@ typedef struct
 #define ACS_ACR33U_A3_3SAM_ICC_READER	0x072f8307
 #define ACS_ACR33U_4SAM_ICC_READER		0x072f8301
 #define ACS_ACR39U_SAM_ICC_READER		0x072fb104
+#define ACS_ACR39_FW_UPGRADE			0x072fb500
 #define ACS_ACR40T_ICC_READER			0x072fb501
 #define ACS_ACR83U						0x072f90d2
 #define ACS_ACR85_PINPAD_READER_ICC		0x072f8306
@@ -302,6 +306,7 @@ typedef struct
 #define ACS_ACR1251_1S_DUAL_READER		0x072f2242
 #define ACS_ACR1252_1S_CL_READER		0x072f223b
 #define ACS_ACR1252IMP_1S_CL_READER		0x072f2259
+#define ACS_ACR1255U_J1_BL				0x072f2250
 #define ACS_WALLETMATE_1S_CL_READER		0x072f226b
 #define ACS_WALLETMATE_II_1S_CL_READER	0x072f2307
 #define ACS_WALLETMATE_II_2S_CL_READER	0x072f2401
