@@ -1,7 +1,7 @@
 /*
     ccid_ifdhandler.h: non-generic ifdhandler functions
     Copyright (C) 2004-2010   Ludovic Rousseau
-    Copyright (C) 2010-2017   Advanced Card Systems Ltd.
+    Copyright (C) 2010-2018   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -56,6 +56,9 @@
 // ACR128 I/O control code for escape command
 #define IOCTL_ACR128_READER_COMMAND	SCARD_CTL_CODE(2079)
 
+/* Control code for toggling the card state in SAM slot */
+#define IOCTL_SMARTCARD_TOGGLE_CARD_STATE	SCARD_CTL_CODE(2075)
+
 /* Control codes for Windows compatibility. */
 #define WINSCARD_CTL_CODE(code)					(0x310000 | (code << 2))
 #define WINIOCTL_CCID_ESCAPE					WINSCARD_CTL_CODE(3500)
@@ -65,6 +68,7 @@
 #define WINIOCTL_SMARTCARD_SET_CARD_TYPE		WINSCARD_CTL_CODE(2060)
 #define WINIOCTL_SMARTCARD_SET_CARD_VOLTAGE		WINSCARD_CTL_CODE(2074)
 #define WINIOCTL_ACR128_READER_COMMAND			WINSCARD_CTL_CODE(2079)
+#define WINIOCTL_SMARTCARD_TOGGLE_CARD_STATE	WINSCARD_CTL_CODE(2075)
 
 #define DRIVER_OPTION_CCID_EXCHANGE_AUTHORIZED 1
 #define DRIVER_OPTION_GEMPC_TWIN_KEY_APDU 2
