@@ -124,8 +124,11 @@ typedef struct
 	pthread_mutex_t *pTransferLock;
 #endif
 
+	// Max packet size of bulk in endpoint
+	int bulk_in_max_packet_size;
+
 	// Max packet size of bulk out endpoint
-	int bulkOutMaxPacketSize;
+	int bulk_out_max_packet_size;
 
 	// For a workaround for ACR122U
 	int last_write_size;
@@ -947,14 +950,14 @@ again:
 					usbDevice[reader_index].ccid.dwDefaultClock = dw2i(device_descriptor, 10);
 					usbDevice[reader_index].ccid.dwMaxDataRate = dw2i(device_descriptor, 23);
 
-					// Fix ACR1222 incorrect max slot index
+					// Fix the incorrect bMaxSlotIndex value for readers.
 					if ((readerID == ACS_ACR1222_1SAM_PICC_READER) ||
 						(readerID == ACS_ACR1222_DUAL_READER))
 						usbDevice[reader_index].ccid.bMaxSlotIndex = 1;
 					else if (readerID == ACS_ACR1222_1SAM_DUAL_READER)
 						usbDevice[reader_index].ccid.bMaxSlotIndex = 2;
-					// Fix ACR32 incorrect max slot index
-					else if (readerID == ACS_ACR32_ICC_READER)
+					else if ((readerID == ACS_ACR32_ICC_READER)
+						|| (readerID == ACS_ACR1255U_J1_BL))
 						usbDevice[reader_index].ccid.bMaxSlotIndex = 0;
 					else
 						usbDevice[reader_index].ccid.bMaxSlotIndex = device_descriptor[4];
@@ -1064,6 +1067,9 @@ again:
 				// Initialize isSamSlot
 				usbDevice[reader_index].ccid.isSamSlot = FALSE;
 
+				/* Initialize isSamNotified. */
+				usbDevice[reader_index].ccid.isSamNotified = FALSE;
+
 				/* Initialize the write delay. */
 				usbDevice[reader_index].ccid.writeDelay = 0;
 
@@ -1234,8 +1240,8 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 		pos = 0;
 		while (length > 0)
 		{
-			if (length > usbDevice[reader_index].bulkOutMaxPacketSize)
-				len = usbDevice[reader_index].bulkOutMaxPacketSize;
+			if (length > usbDevice[reader_index].bulk_out_max_packet_size)
+				len = usbDevice[reader_index].bulk_out_max_packet_size;
 			else
 				len = length;
 
@@ -1265,7 +1271,7 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 
 			/* Delay the write operation. */
 			if ((usbDevice[reader_index].ccid.writeDelay > 0)
-				&& (length > usbDevice[reader_index].bulkOutMaxPacketSize))
+				&& (length > usbDevice[reader_index].bulk_out_max_packet_size))
 			{
 				usleep(usbDevice[reader_index].ccid.writeDelay * 1000);
 			}
@@ -1307,11 +1313,15 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 	unsigned char *buffer)
 {
+	status_t status = STATUS_UNSUCCESSFUL;
 	int rv;
 	int actual_length;
 	char debug_header[] = "<- 121234 ";
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	int duplicate_frame = 0;
+	int packet_size = usbDevice[reader_index].bulk_in_max_packet_size;
+	unsigned char *tmp_buffer = NULL;
+	unsigned int tmp_buffer_length = 0;
 
 	(void)snprintf(debug_header, sizeof(debug_header), "<- %06X ",
 		(int)reader_index);
@@ -1379,56 +1389,93 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 		// Insufficient buffer
 		if (readLen > bufferLen)
 		{
+			*length = 0;
 			DEBUG_CRITICAL("Insufficient buffer");
-			return STATUS_UNSUCCESSFUL;
+			return STATUS_INSUFFICIENT_BUFFER;
 		}
 
 		return STATUS_SUCCESS;
 	}
 
+	/* Calculate the temporary buffer length. */
+	tmp_buffer_length = (*length + packet_size - 1) / packet_size * packet_size;
+
+	/* Allocate the temporary buffer. */
+	tmp_buffer = (unsigned char *) malloc(tmp_buffer_length);
+	if (tmp_buffer == NULL)
+		goto cleanup;
+
 read_again:
 	rv = libusb_bulk_transfer(usbDevice[reader_index].dev_handle,
-		usbDevice[reader_index].bulk_in, buffer, *length,
+		usbDevice[reader_index].bulk_in, tmp_buffer, tmp_buffer_length,
 		&actual_length, usbDevice[reader_index].ccid.readTimeout);
 
 	if (rv < 0)
 	{
-		*length = 0;
 		DEBUG_CRITICAL5("read failed (%d/%d): %d %s",
 			usbDevice[reader_index].bus_number,
 			usbDevice[reader_index].device_address, rv, libusb_error_name(rv));
 
 		if (LIBUSB_ERROR_NO_DEVICE == rv)
-			return STATUS_NO_SUCH_DEVICE;
+			status = STATUS_NO_SUCH_DEVICE;
 
-		return STATUS_UNSUCCESSFUL;
+		goto cleanup;
 	}
 
-	*length = actual_length;
-
-	DEBUG_XXD(debug_header, buffer, *length);
+	DEBUG_XXD(debug_header, tmp_buffer, actual_length);
 
 #define BSEQ_OFFSET 6
-	if ((*length >= BSEQ_OFFSET +1)
-		&& (buffer[BSEQ_OFFSET] < *ccid_descriptor->pbSeq -1))
+	if ((tmp_buffer_length >= BSEQ_OFFSET +1)
+		&& (tmp_buffer[BSEQ_OFFSET] < *ccid_descriptor->pbSeq -1))
 	{
 		// Workaround for ACR122U reader
 		if ((ccid_descriptor->readerID == ACS_ACR122U) &&
 			(usbDevice[reader_index].last_write_size > 64) &&
-			(buffer[BSEQ_OFFSET] == *ccid_descriptor->pbSeq -2))
-			return STATUS_SUCCESS;
+			(tmp_buffer[BSEQ_OFFSET] == *ccid_descriptor->pbSeq -2))
+		{
+			status = STATUS_SUCCESS;
+			goto cleanup;
+		}
 
 		duplicate_frame++;
 		if (duplicate_frame > 10)
 		{
 			DEBUG_CRITICAL("Too many duplicate frame detected");
-			return STATUS_UNSUCCESSFUL;
+			goto cleanup;
 		}
 		DEBUG_INFO1("Duplicate frame detected");
 		goto read_again;
 	}
 
-	return STATUS_SUCCESS;
+	status = STATUS_SUCCESS;
+
+cleanup:
+	if (status != STATUS_SUCCESS)
+		*length = 0;
+	else
+	{
+		if (*length < actual_length)
+		{
+			*length = 0;
+			DEBUG_CRITICAL("Insufficient buffer");
+			status = STATUS_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			/* Return the buffer. */
+			*length = actual_length;
+			memcpy(buffer, tmp_buffer, *length);
+		}
+	}
+
+	/* Free the temporary buffer. */
+	if (tmp_buffer != NULL)
+	{
+		free(tmp_buffer);
+		tmp_buffer = NULL;
+	}
+
+	return status;
 } /* ReadUSB */
 
 
@@ -1613,7 +1660,8 @@ static int get_end_points(struct libusb_config_descriptor *desc,
 	usbdevice->bulk_out = 0;
 	usbdevice->interrupt = 0;
 	usbdevice->has_interrupt = FALSE;
-	usbdevice->bulkOutMaxPacketSize = 0;
+	usbdevice->bulk_in_max_packet_size = 0;
+	usbdevice->bulk_out_max_packet_size = 0;
 
 	/*
 	 * 3 Endpoints maximum: Interrupt In, Bulk In, Bulk Out
@@ -1639,13 +1687,18 @@ static int get_end_points(struct libusb_config_descriptor *desc,
 
 		if ((bEndpointAddress & LIBUSB_ENDPOINT_DIR_MASK)
 			== LIBUSB_ENDPOINT_IN)
+		{
 			usbdevice->bulk_in = bEndpointAddress;
+			usbdevice->bulk_in_max_packet_size =
+				usb_interface->altsetting->endpoint[i].wMaxPacketSize;
+		}
 
 		if ((bEndpointAddress & LIBUSB_ENDPOINT_DIR_MASK)
 			== LIBUSB_ENDPOINT_OUT)
 		{
 			usbdevice->bulk_out = bEndpointAddress;
-			usbdevice->bulkOutMaxPacketSize = usb_interface->altsetting->endpoint[i].wMaxPacketSize;
+			usbdevice->bulk_out_max_packet_size =
+				usb_interface->altsetting->endpoint[i].wMaxPacketSize;
 		}
 	}
 
@@ -2152,7 +2205,8 @@ static void *Multi_PollingProc(void *p_ext)
 									usbDevice[msExt->reader_index].ccid.bStatus[i] = CCID_ICC_PRESENT_ACTIVE;
 
 									/* Update the status for SAM slot. */
-									if ((i == 0) && (usbDevice[msExt->reader_index].ccid.isSamSlot))
+									if (usbDevice[msExt->reader_index].ccid.isSamSlot
+										&& usbDevice[msExt->reader_index].ccid.isSamNotified)
 									{
 										usbDevice[msExt->reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 									}
@@ -2162,7 +2216,8 @@ static void *Multi_PollingProc(void *p_ext)
 									usbDevice[msExt->reader_index].ccid.bStatus[i] = CCID_ICC_ABSENT;
 
 									/* Update the status for SAM slot. */
-									if ((i == 0) && (usbDevice[msExt->reader_index].ccid.isSamSlot))
+									if (usbDevice[msExt->reader_index].ccid.isSamSlot
+										&& usbDevice[msExt->reader_index].ccid.isSamNotified)
 									{
 										usbDevice[msExt->reader_index].ccid.dwSlotStatus = IFD_ICC_NOT_PRESENT;
 									}
@@ -2583,7 +2638,8 @@ static void *CardDetectionThread(void *pParam)
 									usbDevice[reader_index].ccid.bStatus[i] = CCID_ICC_PRESENT_ACTIVE;
 
 									/* Update the status for SAM slot. */
-									if ((i == 0) && (usbDevice[reader_index].ccid.isSamSlot))
+									if (usbDevice[reader_index].ccid.isSamSlot
+										&& usbDevice[reader_index].ccid.isSamNotified)
 									{
 										usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 									}
@@ -2593,7 +2649,8 @@ static void *CardDetectionThread(void *pParam)
 									usbDevice[reader_index].ccid.bStatus[i] = CCID_ICC_ABSENT;
 
 									/* Update the status for SAM slot. */
-									if ((i == 0) && (usbDevice[reader_index].ccid.isSamSlot))
+									if (usbDevice[reader_index].ccid.isSamSlot
+										&& usbDevice[reader_index].ccid.isSamNotified)
 									{
 										usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_NOT_PRESENT;
 									}
