@@ -41,7 +41,7 @@
  * Not exported funtions declaration
  */
 
-static bool PPS_Match (BYTE * request, unsigned len_request, BYTE * reply, unsigned len_reply);
+static int PPS_Match (BYTE * request, unsigned len_request, BYTE * reply, unsigned len_reply);
 
 static unsigned PPS_GetLength (BYTE * block);
 
@@ -91,7 +91,7 @@ PPS_Exchange (int lun, BYTE * params, unsigned *length, unsigned char *pps1)
   return ret;
 }
 
-static bool
+static int
 PPS_Match (BYTE * request, unsigned len_request, BYTE * confirm, unsigned len_confirm)
 {
   /* See if the reply differs from request */
@@ -103,7 +103,9 @@ PPS_Match (BYTE * request, unsigned len_request, BYTE * confirm, unsigned len_co
     return FALSE;
 
   /* See if the card specifies other than default FI and D */
-  if ((PPS_HAS_PPS1 (confirm)) && (confirm[2] != request[2]))
+  if ((PPS_HAS_PPS1 (confirm))
+		&& (len_confirm > 2)
+		&& (confirm[2] != request[2]))
     return FALSE;
 
   return TRUE;
